@@ -1,9 +1,10 @@
 import abc
-from typing import Any, Iterable, Mapping, Sequence, Union, Generator, Tuple
+from typing import Any, Iterable, Mapping, Sequence, Union, Generator, Tuple, Set
 
 import pandas as pd
 from pandas.core.frame import DataFrame as _InternalDataFrame
-from natsort import natsorted, ns
+from natsort import natsorted
+from typeddfs.utils import Utils
 
 from typeddfs._pretty_dfs import PrettyDf
 from typeddfs.df_errors import NoValueError, UnsupportedOperationError, ValueNotUniqueError
@@ -37,6 +38,16 @@ class CoreDf(PrettyDf, metaclass=abc.ABCMeta):
         """
         raise NotImplementedError()
 
+    def strip_control_chars(self) -> __qualname__:
+        """
+        Removes all control characters (Unicode group 'C') from all string-typed columns.
+        """
+        df = self.vanilla_reset()
+        for c in df.columns:
+            if Utils.is_string_dtype(df[c]):
+                df[c] = df[c].map(Utils.strip_control_chars)
+        return self.__class__._convert_typed(df)
+
     def iter_row_col(self) -> Generator[Tuple[Tuple[int, int], Any], None, None]:
         """
         Iterates over ``((row, col), value)`` tuples.
@@ -46,23 +57,24 @@ class CoreDf(PrettyDf, metaclass=abc.ABCMeta):
             for col in range(len(self.columns)):
                 yield (row, col), self.iat[row, col]
 
-    def only(self, column: str, exclude_na: bool = False) -> Any:
+    def only(self, column: str, exclude_na: bool = False, show_vals: bool = False) -> Any:
         """
         Returns the single unique value in a column.
         Raises an error if zero or more than one value is in the column.
 
         Args:
             column: The name of the column
-            exclude_na: Exclude null values
+            exclude_na: Exclude None/pd.NA values
+            show_vals: If there are multiple unique values, list them all in the error message
         """
         x = set(self[column].unique())
         if exclude_na:
             x = {k for k in x if not pd.isna(k)}
         if len(x) > 1:
-            raise ValueNotUniqueError(f"Multiple values for {column}")
+            raise ValueNotUniqueError(f"Multiple values for {column}", key=column, values=set(x))
         if len(x) == 0:
             raise NoValueError(
-                f"No values for {column}" + " (excluding null)" if exclude_na else ""
+                f"No values for {column}" + " (excluding null)" if exclude_na else "", key=column
             )
         return next(iter(x))
 
@@ -77,50 +89,82 @@ class CoreDf(PrettyDf, metaclass=abc.ABCMeta):
             cols = [cols]
         return self.__class__._change(self[cols + [c for c in self.columns if c not in cols]])
 
-    def sort_natural(self, column: str, alg: int = ns.INT) -> __qualname__:
+    def sort_natural(
+        self, column: str, alg: Union[None, int, Set[str]] = None, reverse: bool = False
+    ) -> __qualname__:
         """
         Calls ``natsorted`` on a single column.
 
         Args:
             column: The name of the (single) column to sort by
             alg: Input as the ``alg`` argument to ``natsorted``
+                 If ``None``, the "best" algorithm is chosen from the dtype of ``column``
+                 via :py.meth:`typeddfs.utils.Utils.guess_natsort_alg`.
+                 Otherwise, :py.meth:typeddfs.utils.Utils.exact_natsort_alg`
+                 is called with ``Utils.exact_natsort_alg(alg)``.
+            reverse: Reverse the sort order (e.g. 'z' before 'a')
         """
         df = self.vanilla_reset()
-        zzz = natsorted([s for s in df[column]], alg=alg)
+        if alg is None:
+            _, alg = Utils.guess_natsort_alg(self[column].dtype)
+        else:
+            _, alg = Utils.exact_natsort_alg(alg)
+        zzz = natsorted([s for s in df[column]], alg=alg, reverse=reverse)
         df["__sort"] = df[column].map(lambda s: zzz.index(s))
         df.__class__ = self.__class__
         df = df.sort_values("__sort").drop_cols(["__sort"])
         return self.__class__._change(df)
 
-    def sort_natural_index(self, alg: int = ns.INT) -> __qualname__:
+    def sort_natural_index(self, alg: int = None, reverse: bool = False) -> __qualname__:
         """
         Calls natsorted on this index. Works for multi-index too.
 
         Args:
             alg: Input as the ``alg`` argument to ``natsorted``
+                 If ``None``, the "best" algorithm is chosen from the dtype of ``column``
+                 via :py.meth:`typeddfs.utils.Utils.guess_natsort_alg`.
+                 Otherwise, :py.meth:typeddfs.utils.Utils.exact_natsort_alg`
+                 is called with ``Utils.exact_natsort_alg(alg)``.
+            reverse: Reverse the sort order (e.g. 'z' before 'a')
         """
         df = self.copy()
+        if alg is None:
+            # TODO: Does this work for multi-index?
+            _, alg = Utils.guess_natsort_alg(self.index.dtype)
+        else:
+            _, alg = Utils.exact_natsort_alg(alg)
         zzz = natsorted([s for s in df.index], alg=alg)
         df["__sort"] = df.index.map(lambda s: zzz.index(s))
         df.__class__ = self.__class__
         df = df.sort_values("__sort").drop_cols(["__sort"])
         return self.__class__._change(df)
 
-    def drop_cols(self, cols: Union[str, Iterable[str]]) -> __qualname__:
+    def drop_cols(self, *cols: Union[str, Iterable[str]]) -> __qualname__:
         """
         Drops columns, ignoring those that are not present.
 
         Args:
             cols: A single column name or a list of column names
         """
-        df = self.copy()
-        if isinstance(cols, str):
-            cols = [cols]
-        for c in cols:
-            if c in self.columns:
+        my_cols = set()
+        for cols_ in cols:
+            if isinstance(cols_, str):
+                my_cols.add(cols_)
+            else:
+                my_cols.update(cols_)
+        df = self
+        for c in my_cols:
+            if c in df.columns:
                 df = df.drop(c, axis=1)
         return self.__class__._change(df)
 
+    def rename_cols(self, **cols) -> __qualname__:
+        """
+        Shorthand for ``.rename(columns=)``.
+        """
+        df = self.rename(columns=cols)
+        return self.__class__._change(df)
+
     def vanilla_reset(self) -> pd.DataFrame:
         """
         Same as :py.meth:`vanilla`, but resets the index -- but dropping the index if it has no name.
@@ -263,15 +307,15 @@ class CoreDf(PrettyDf, metaclass=abc.ABCMeta):
         return self.__class__._change(df)
 
     # noinspection PyFinal
-    def ffill(self, *args, **kwargs) -> __qualname__:
+    def ffill(self, **kwargs) -> __qualname__:
         self._no_inplace(kwargs)
-        df = super().ffill(*args, **kwargs)
+        df = super().ffill(**kwargs)
         return self.__class__._change(df)
 
     # noinspection PyFinal
-    def bfill(self, *args, **kwargs) -> __qualname__:
+    def bfill(self, **kwargs) -> __qualname__:
         self._no_inplace(kwargs)
-        df = super().bfill(*args, **kwargs)
+        df = super().bfill(**kwargs)
         return self.__class__._change(df)
 
     # noinspection PyFinal
