@@ -18,6 +18,7 @@ from tabulate import TableFormat, tabulate
 
 from typeddfs._core_dfs import CoreDf
 from typeddfs._utils import _FAKE_SEP, PathLike
+from typeddfs.checksums import Checksums
 from typeddfs.df_errors import (
     NonStrColumnError,
     NotSingleColumnError,
@@ -61,7 +62,11 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
 
     @classmethod
     def read_file(
-        cls, path: Union[Path, str], check_hash: Union[None, bool, str, PurePath] = False
+        cls,
+        path: Union[Path, str],
+        file_hash: Optional[bool] = None,
+        dir_hash: Optional[bool] = None,
+        hex_hash: Optional[str] = None,
     ) -> __qualname__:
         """
         Reads from a file (or possibly URL), guessing the format from the filename extension.
@@ -88,21 +93,18 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
         Args:
             path: Only path-like strings or pathlib objects are supported, not buffers
                   (because we need a filename).
-            check_hash: Require a hash file for the file or directory and make sure it matches.
-                        Must be one of these:
-                          - False / "no" -- do not check
-                          - True / "yes" -- check either a file hash or a dir hash
-                          - None -- Treat as "yes" if dictated by ``self.get_typing().io``
-                          - "file" -- check a file hash (ignoring the filename in the hash file)
-                          - "dir" -- check a dir hash (looking up the filename in the hash file)
-                          - str or Path (extant); a path to the hash file (does not check the filename)
-                          - str matching [A-Ha-h0-9]+; a hex-encoded digest
+            file_hash: Check against a hash file specific to this file (e.g. <path>.sha1)
+            dir_hash: Check against a per-directory hash file
+            hex_hash: Check against this hex-encoded hash
 
         Returns:
             An instance of this class
         """
         path = Path(path)
-        cls._check_hash(path, check_hash)
+        algorithm = cls.get_typing().io.hash_algorithm
+        Checksums.verify_any(
+            path, file_hash=file_hash, dir_hash=dir_hash, computed=hex_hash, algorithm=algorithm
+        )
         df = cls._call_read(cls, path)
         return cls._convert_typed(df)
 
@@ -163,7 +165,7 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
         z = self._call_write(path)
         file_hash = file_hash is True or file_hash is None and self.get_typing().io.file_hash
         dir_hash = dir_hash is True or dir_hash is None and self.get_typing().io.dir_hash
-        Utils.add_any_hashes(
+        Checksums.add_any_hashes(
             path,
             to_file=file_hash,
             to_dir=dir_hash,
@@ -275,7 +277,9 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
                 **kwargs,
             )
         except pd.errors.EmptyDataError:
-            df = pd.DataFrame()
+            # TODO: Figure out what EmptyDataError means
+            # df = pd.DataFrame()
+            return cls.new_df()
         if len(df.columns) > 1:
             raise NotSingleColumnError(f"Read multiple columns on {path_or_buff}")
         return cls._convert_typed(df)
@@ -285,8 +289,9 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
         try:
             return cls._convert_typed(pd.read_fwf(*args, **kwargs))
         except pd.errors.EmptyDataError:
-            df = pd.DataFrame()
-        return cls._convert_typed(df)
+            # TODO: Figure out what EmptyDataError means
+            # df = pd.DataFrame()
+            return cls.new_df()
 
     def to_fwf(
         self,
@@ -360,7 +365,7 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
             return content
         _encoding = dict(encoding=kwargs.get("encoding")) if "encoding" in kwargs else {}
         _compression = dict(encoding=kwargs.get("compression")) if "compression" in kwargs else {}
-        Utils.write(path_or_buff, content, mode, **_encoding, **_compression)
+        Utils.write(path_or_buff, content, mode=mode, **_encoding, **_compression)
 
     @classmethod
     def read_flexwf(
@@ -400,7 +405,9 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
                 **kwargs,
             )
         except pd.errors.EmptyDataError:
-            df = pd.DataFrame()
+            # TODO: Figure out what EmptyDataError means
+            # df = pd.DataFrame()
+            return cls.new_df()
         df.columns = [c.strip() for c in df.columns]
         for c in df.columns:
             try:
@@ -428,14 +435,16 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
         content = self._tabulate(fmt, disable_numparse=True)
         if path_or_buff is None:
             return content
-        Utils.write(path_or_buff, content, mode, **kwargs)
+        Utils.write(path_or_buff, content, mode=mode, **kwargs)
 
     @classmethod
     def read_excel(cls, io, sheet_name: _SheetNamesOrIndices = 0, *args, **kwargs) -> __qualname__:
         try:
             df = pd.read_excel(io, sheet_name, *args, **kwargs)
         except pd.errors.EmptyDataError:
-            df = pd.DataFrame()
+            # TODO: Figure out what EmptyDataError means
+            # df = pd.DataFrame()
+            return cls.new_df()
         # This only applies for .xlsb -- the others don't have this problem
         if "Unnamed: 0" in df.columns:
             df = df.drop("Unnamed: 0", axis=1)
@@ -535,7 +544,9 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
         try:
             df = pd.read_pickle(filepath_or_buffer, *args, **kwargs)
         except pd.errors.EmptyDataError:
-            df = pd.DataFrame()
+            # TODO: Figure out what EmptyDataError means
+            # df = pd.DataFrame()
+            return cls.new_df()
         return cls._convert_typed(df)
 
     # noinspection PyFinal
@@ -548,7 +559,9 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
         try:
             df = pd.read_json(*args, **kwargs)
         except pd.errors.EmptyDataError:
-            df = pd.DataFrame()
+            # TODO: Figure out what EmptyDataError means
+            # df = pd.DataFrame()
+            return cls.new_df()
         return cls._convert_typed(df)
 
     @classmethod
@@ -556,10 +569,14 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
         try:
             df = pd.read_xml(*args, **kwargs)
         except pd.errors.EmptyDataError:
-            df = pd.DataFrame()
+            # TODO: Figure out what EmptyDataError means
+            # df = pd.DataFrame()
+            return cls.new_df()
         # see to_xml for why these fixes are needed
         if "__xml_is_empty_" in df.reset_index().columns:
-            df = pd.DataFrame()
+            # TODO: This ok?
+            # df = pd.DataFrame()
+            return cls.new_df()
         elif "__xml_index_" in df.columns:
             df = df.drop(columns={"__xml_index_"})
         return cls._convert_typed(df)
@@ -597,7 +614,9 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
         try:
             df = pd.read_feather(*args, **kwargs)
         except pd.errors.EmptyDataError:
-            df = pd.DataFrame()
+            # TODO: Figure out what EmptyDataError means
+            # df = pd.DataFrame()
+            return cls.new_df()
         if "__feather_ignore_" in df.columns:
             df = df.drop("__feather_ignore_", axis=1)
         return cls._convert_typed(df)
@@ -637,7 +656,9 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
         try:
             df = pd.read_parquet(*args, **kwargs)
         except pd.errors.EmptyDataError:
-            df = pd.DataFrame()
+            # TODO: Figure out what EmptyDataError means
+            # df = pd.DataFrame()
+            return cls.new_df()
         return cls._convert_typed(df)
 
     # noinspection PyMethodOverriding,PyBroadException,DuplicatedCode
@@ -718,7 +739,9 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
         try:
             df = pd.read_csv(*args, **kwargs)
         except pd.errors.EmptyDataError:
-            df = pd.DataFrame()
+            # TODO: Figure out what EmptyDataError means
+            # df = pd.DataFrame()
+            return cls.new_df()
         return cls._convert_typed(df)
 
     def to_tsv(self, *args, **kwargs) -> Optional[str]:
@@ -744,7 +767,9 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
         try:
             df = pd.read_hdf(*args, key=key, **kwargs)
         except pd.errors.EmptyDataError:
-            df = pd.DataFrame()
+            # TODO: Figure out exactly what EmptyDataError is
+            return cls.new_df()
+        # noinspection PyTypeChecker
         return cls._convert_typed(df)
 
     def to_html(self, *args, **kwargs) -> Optional[str]:
@@ -893,17 +918,6 @@ class AbsDf(CoreDf, metaclass=abc.ABCMeta):
             kwargs["encoding"] = Utils.get_encoding(encoding)
         return kwargs
 
-    @classmethod
-    def _check_hash(cls, path: Path, check_hash: Union[None, bool, str, PurePath]):
-        t = cls.get_typing().io
-        if check_hash is None and t.file_hash:
-            check_hash = "file"
-        elif check_hash is None and t.dir_hash:
-            check_hash = "dir"
-        elif check_hash is None:
-            check_hash = "no"
-        Utils.verify_any_hash(path, check_hash, algorithm=t.hash_algorithm)
-
     @classmethod
     def _lines_files_apply(cls) -> bool:
         return len(cls.get_typing().required_names) <= 1
