@@ -8,6 +8,8 @@ from collections import defaultdict
 from typing import Any, Callable, Optional, Sequence, Type, Union
 
 import pandas as pd
+
+from typeddfs.checksums import Checksums
 from typeddfs.utils import Utils
 
 from typeddfs.base_dfs import BaseDf
@@ -63,13 +65,22 @@ class _GenericBuilder:
         self._secure = False
         self._req_hash: Optional = False
         self._req_order: Optional = False
-        # use utf-8 by default
-        self.encoding()
         # make these use an explicit version
         # the user can override if needed
         self.add_read_kwargs("pickle", protocol=5)
         self.add_write_kwargs("pickle", protocol=5)
 
+    def doc(self, s: str) -> __qualname__:
+        """
+        Sets the docstring.
+        This has the same effect as setting doc in :py.meth:`__init__`.
+
+        Returns:
+            This builder for chaining
+        """
+        self._doc = s
+        return self
+
     def series_names(
         self, index: Union[None, bool, str] = False, columns: Union[None, bool, str] = False
     ) -> __qualname__:
@@ -189,7 +200,7 @@ class _GenericBuilder:
         Returns:
             This builder for chaining
         """
-        self._hash_alg = Utils.get_algorithm(alg)
+        self._hash_alg = Checksums.get_algorithm(alg)
         self._hash_file = file
         self._hash_dir = directory
         return self
@@ -273,7 +284,7 @@ class _GenericBuilder:
             raise DfTypeConstructionError(f"Hash algorithm {self._hash_alg} forbidden by .secure()")
         self._check_final()
 
-        _io_typing = IoTyping(
+        _io_typing = IoTyping[BaseDf](
             _remap_suffixes=dict(self._remapped_suffixes),
             _text_encoding=self._encoding,
             _read_kwargs=dict(self._read_kwargs),
@@ -596,7 +607,7 @@ class TypedDfBuilder(_GenericBuilder):
             raise ClashError(f"{','.join(_FORBIDDEN_NAMES)} are forbidden names")
         for name in names:
             if name in [*self._req_cols, *self._req_meta, *self._res_cols, *self._res_meta]:
-                raise ClashError(f"Column {name} for {self._name} already exists")
+                raise ClashError(f"Column {name} for {self._name} already exists", keys={name})
 
 
 __all__ = ["TypedDfBuilder", "MatrixDfBuilder", "AffinityMatrixDfBuilder"]
